#Message on change fetches data from a website, and notifies the user when the data has changed.
#Copyright (C) 2024  Rūdolfs Driķis
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

import requests
import argparse
import time
from playsound import playsound
import webbrowser


def main(url='', delay=10, change_sound='change.mp3', open_page=False):
    comparison_data = requests.get(url).text
    while True:
        latest_obtained_data = requests.get(url).text
        if latest_obtained_data == comparison_data:
            print('Nothing has changed')
        else:
            break

        time.sleep(delay)

    print("Something has changed!")
    if open_page:
        webbrowser.open(url)
    playsound(change_sound)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'app.ui'
##
## Created by: Qt User Interface Compiler version 6.6.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QGridLayout, QLabel,
    QLineEdit, QMainWindow, QPushButton, QSizePolicy,
    QSpacerItem, QSpinBox, QStatusBar, QVBoxLayout,
    QWidget)


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(289, 308)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.gridLayout_2 = QGridLayout(self.centralwidget)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.label = QLabel(self.centralwidget)
        self.label.setObjectName(u"label")
        self.label.setEnabled(True)

        self.verticalLayout_2.addWidget(self.label)

        self.lineEdit = QLineEdit(self.centralwidget)
        self.lineEdit.setObjectName(u"lineEdit")

        self.verticalLayout_2.addWidget(self.lineEdit)

        self.checkBox = QCheckBox(self.centralwidget)
        self.checkBox.setObjectName(u"checkBox")

        self.verticalLayout_2.addWidget(self.checkBox)

        self.lineEdit_2 = QLineEdit(self.centralwidget)
        self.lineEdit_2.setObjectName(u"lineEdit_2")

        self.verticalLayout_2.addWidget(self.lineEdit_2)

        self.pushButton_2 = QPushButton(self.centralwidget)
        self.pushButton_2.setObjectName(u"pushButton_2")

        self.verticalLayout_2.addWidget(self.pushButton_2)

        self.checkBox_3 = QCheckBox(self.centralwidget)
        self.checkBox_3.setObjectName(u"checkBox_3")

        self.verticalLayout_2.addWidget(self.checkBox_3)

        self.checkBox_2 = QCheckBox(self.centralwidget)
        self.checkBox_2.setObjectName(u"checkBox_2")

        self.verticalLayout_2.addWidget(self.checkBox_2)

        self.spinBox = QSpinBox(self.centralwidget)
        self.spinBox.setObjectName(u"spinBox")
        self.spinBox.setEnabled(True)

        self.verticalLayout_2.addWidget(self.spinBox)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.pushButton = QPushButton(self.centralwidget)
        self.pushButton.setObjectName(u"pushButton")

        self.verticalLayout_2.addWidget(self.pushButton)


        self.gridLayout_2.addLayout(self.verticalLayout_2, 1, 0, 1, 1)

        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.label.setText(QCoreApplication.translate("MainWindow", u"Specify your url", None))
        self.checkBox.setText(QCoreApplication.translate("MainWindow", u"Custom audio files", None))
        self.pushButton_2.setText(QCoreApplication.translate("MainWindow", u"Select custom", None))
        self.checkBox_3.setText(QCoreApplication.translate("MainWindow", u"Seperate first time and repeeted audio files", None))
        self.checkBox_2.setText(QCoreApplication.translate("MainWindow", u"Cusom delay", None))
        self.pushButton.setText(QCoreApplication.translate("MainWindow", u"Start", None))
    # retranslateUi




# Message on change fetches data from a website, and notifies the user when the data has changed.
# Copyright (C) 2024  Rūdolfs Driķis
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import sys
import threading

from PySide6.QtWidgets import QApplication, QMainWindow, QVBoxLayout, QPushButton, QStackedWidget, QWidget
from message_on_change.gui import Ui_MainWindow as Ui_File1
from message_on_change.status import Ui_InfoWindow as Ui_File2
import message_on_change.fetch_logic
from threading import Thread


class MainWindow(QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()

        self.stacked_widget = QStackedWidget(self)

        # Create instances of the compiled UI classes
        self.ui_file1 = Ui_File1()
        self.ui_file2 = Ui_File2()

        # Set up UI from file1
        self.ui_file1.setupUi(self)
        self.stacked_widget.addWidget(self.ui_file1.centralwidget)

        # Hide the custom delay input box and add the switch visibility function
        self.ui_file1.spinBox.hide()
        self.ui_file1.spinBox.setEnabled(True)
        self.ui_file1.checkBox_2.clicked.connect(self.toggle_widget_visibility)

        # Custom file input box toggle
        self.ui_file1.pushButton_2.hide()
        self.ui_file1.lineEdit_2.hide()
        self.ui_file1.checkBox.clicked.connect(self.toggle_file_select_visibility)

        # file select logic
        self.ui_file1.pushButton_2.clicked.connect(self.openFile)

        # call the main function
        self.ui_file1.pushButton.clicked.connect(self.startProcess)

        # Set up UI from file2
        self.ui_file2.setupUi(self)
        self.stacked_widget.addWidget(self.ui_file2.centralwidget)

        self.current_index = 0

        # self.switch_button = QPushButton('Switch Layout', self)
        # self.switch_button.clicked.connect(self.switch_layout)

        self.central_widget = QWidget(self)
        self.setCentralWidget(self.central_widget)

        self.layout = QVBoxLayout(self.central_widget)
        # self.layout.addWidget(self.switch_button)
        self.layout.addWidget(self.stacked_widget)
        self.layout.addStretch(1)  # Add stretch to push the button to the top

        # Initially, show the first layout
        self.stacked_widget.setCurrentIndex(0)

    def switch_layout(self):
        self.current_index = (self.current_index + 1) % self.stacked_widget.count()
        self.stacked_widget.setCurrentIndex(self.current_index)

    def getOptions(self):
        # options to pass
        return {
            'url': self.ui_file1.lineEdit.text(),
            'customFile': self.ui_file1.checkBox.isChecked(),
            'customFilePath': self.ui_file1.lineEdit_2.text(),
            'separateFile': self.ui_file1.checkBox_3.isChecked(),
            'customDelay': self.ui_file1.checkBox_2.isChecked(),
            'customDelayValue': self.ui_file1.spinBox.value()
        }

    def toggle_widget_visibility(self):
        # Show or hide the widget based on the checkbox state
        if self.ui_file1.checkBox_2.isChecked():
            self.ui_file1.spinBox.show()
        else:
            self.ui_file1.spinBox.hide()

    def toggle_file_select_visibility(self):
        if self.ui_file1.checkBox.isChecked():
            self.ui_file1.pushButton_2.show()
            self.ui_file1.lineEdit_2.show()
        else:
            self.ui_file1.pushButton_2.hide()
            self.ui_file1.lineEdit_2.hide()

    def openFile(self):
        options = QFileDialog.Options()
        options |= QFileDialog.DontUseNativeDialog

        file_dialog = QFileDialog()
        file_path, _ = file_dialog.getOpenFileName(self, "Select Sound File", "", "(*.wav *.mp3);;All Files (*)",
                                                   options=options)

        self.ui_file1.lineEdit_2.setText(file_path)

    def startProcess(self, options):
        print('process started')
        options = self.getOptions()
        proc = threading.Thread(target=message_on_change.fetch_logic.main, kwargs={
            'url': options['url'],
            'delay': options['customDelayValue'],
            'change_sound': options['customFilePath']
        })

        try:
            proc.start()
            self.switch_layout()

        except:
            print('specify needed arguments')


def main():
    app = QApplication(sys.argv)
    main_window = MainWindow()
    main_window.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'stats.ui'
##
## Created by: Qt User Interface Compiler version 6.6.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QGridLayout, QLabel, QMainWindow,
    QPushButton, QSizePolicy, QSpacerItem, QStatusBar,
    QVBoxLayout, QWidget)

class Ui_InfoWindow(object):
    def setupUi(self, InfoWindow):
        if not InfoWindow.objectName():
            InfoWindow.setObjectName(u"InfoWindow")
        InfoWindow.resize(322, 308)
        self.centralwidget = QWidget(InfoWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.gridLayout_2 = QGridLayout(self.centralwidget)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setSpacing(6)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.label_4 = QLabel(self.centralwidget)
        self.label_4.setObjectName(u"label_4")

        self.verticalLayout_2.addWidget(self.label_4)

        self.label_6 = QLabel(self.centralwidget)
        self.label_6.setObjectName(u"label_6")

        self.verticalLayout_2.addWidget(self.label_6)

        self.label_2 = QLabel(self.centralwidget)
        self.label_2.setObjectName(u"label_2")

        self.verticalLayout_2.addWidget(self.label_2)

        self.label_3 = QLabel(self.centralwidget)
        self.label_3.setObjectName(u"label_3")

        self.verticalLayout_2.addWidget(self.label_3)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.label = QLabel(self.centralwidget)
        self.label.setObjectName(u"label")
        self.label.setEnabled(True)

        self.verticalLayout_2.addWidget(self.label)

        self.label_5 = QLabel(self.centralwidget)
        self.label_5.setObjectName(u"label_5")

        self.verticalLayout_2.addWidget(self.label_5)

        self.pushButton = QPushButton(self.centralwidget)
        self.pushButton.setObjectName(u"pushButton")

        self.verticalLayout_2.addWidget(self.pushButton)


        self.gridLayout_2.addLayout(self.verticalLayout_2, 1, 0, 1, 1)

        InfoWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QStatusBar(InfoWindow)
        self.statusbar.setObjectName(u"statusbar")
        InfoWindow.setStatusBar(self.statusbar)

        self.retranslateUi(InfoWindow)

        QMetaObject.connectSlotsByName(InfoWindow)
    # setupUi

    def retranslateUi(self, InfoWindow):
        InfoWindow.setWindowTitle(QCoreApplication.translate("InfoWindow", u"MainWindow", None))
        self.label_4.setText(QCoreApplication.translate("InfoWindow", u"You are compareing the webpage on:", None))
        self.label_6.setText(QCoreApplication.translate("InfoWindow", u"<URL>", None))
        self.label_2.setText(QCoreApplication.translate("InfoWindow", u"Once a change occours the <soundfile> will be played", None))
        self.label_3.setText(QCoreApplication.translate("InfoWindow", u"upon a change the url <will/will not> be opened", None))
        self.label.setText(QCoreApplication.translate("InfoWindow", u"The process has been going on for <time>", None))
        self.label_5.setText(QCoreApplication.translate("InfoWindow", u"You've sent <requestcont> requests since <starttime>", None))
        self.pushButton.setText(QCoreApplication.translate("InfoWindow", u"Stop process", None))
    # retranslateUi




#Message on change fetches data from a website, and notifies the user when the data has changed.
#Copyright (C) 2024  Rūdolfs Driķis
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

import argparse
import message_on_change.fetch_logic


parser = argparse.ArgumentParser(
    prog="Comparator",
    description="compares a downloaded version of a webpage, to the hosted one on a specified URL at a given interval, to signal to you when a change has been made."
)
parser.add_argument('URL')
parser.add_argument('-d', '--delay', nargs=1)
parser.add_argument('-o', '--open-after', dest='open_after', required=False, action='store_true')
args = parser.parse_args()

if args.delay is not None:
    delay = args.delay
else:
    delay = 10

if args.open_after is not None:
    open_after = True
else:
    print('open after')
    open_after = False

message_on_change.fetch_logic.main(
    url=args.URL,
    delay=delay,
    open_page=open_after
)